@@ -1,27 +1,21 @@
-from __future__ import division, absolute_import, print_function
-
-import warnings
-
 import numpy as np
-from numpy.testing import (assert_, TestCase, assert_array_equal,
-                           assert_allclose, run_module_suite,
-                           suppress_warnings)
-
-rlevel = 1
+from numpy.testing import (
+    assert_, assert_array_equal, assert_allclose, suppress_warnings
+    )
 
 
-class TestRegression(TestCase):
-    def test_masked_array_create(self,level=rlevel):
+class TestRegression:
+    def test_masked_array_create(self):
         # Ticket #17
         x = np.ma.masked_array([0, 1, 2, 3, 0, 4, 5, 6],
                                mask=[0, 0, 0, 1, 1, 1, 0, 0])
         assert_array_equal(np.ma.nonzero(x), [[1, 2, 6, 7]])
 
-    def test_masked_array(self,level=rlevel):
+    def test_masked_array(self):
         # Ticket #61
         np.ma.array(1, mask=[1])
 
-    def test_mem_masked_where(self,level=rlevel):
+    def test_mem_masked_where(self):
         # Ticket #62
         from numpy.ma import masked_where, MaskType
         a = np.zeros((1, 1))
@@ -29,7 +23,7 @@ class TestRegression(TestCase):
         c = masked_where(b, a)
         a-c
 
-    def test_masked_array_multiply(self,level=rlevel):
+    def test_masked_array_multiply(self):
         # Ticket #254
         a = np.ma.zeros((4, 1))
         a[2, 0] = np.ma.masked
@@ -37,7 +31,7 @@ class TestRegression(TestCase):
         a*b
         b*a
 
-    def test_masked_array_repeat(self, level=rlevel):
+    def test_masked_array_repeat(self):
         # Ticket #271
         np.ma.array([1], mask=False).repeat(10)
 
@@ -77,5 +71,21 @@ class TestRegression(TestCase):
             # ddof should not have an effect (it gets cancelled out)
             assert_allclose(r0.data, r1.data)
 
-if __name__ == "__main__":
-    run_module_suite()
+    def test_mask_not_backmangled(self):
+        # See gh-10314.  Test case taken from gh-3140.
+        a = np.ma.MaskedArray([1., 2.], mask=[False, False])
+        assert_(a.mask.shape == (2,))
+        b = np.tile(a, (2, 1))
+        # Check that the above no longer changes a.shape to (1, 2)
+        assert_(a.mask.shape == (2,))
+        assert_(b.shape == (2, 2))
+        assert_(b.mask.shape == (2, 2))
+
+    def test_empty_list_on_structured(self):
+        # See gh-12464. Indexing with empty list should give empty result.
+        ma = np.ma.MaskedArray([(1, 1.), (2, 2.), (3, 3.)], dtype='i4,f4')
+        assert_array_equal(ma[[]], ma[:0])
+
+    def test_masked_array_tobytes_fortran(self):
+        ma = np.ma.arange(4).reshape((2,2))
+        assert_array_equal(ma.tobytes(order='F'), ma.T.tobytes())
